/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.grammar.token;

import java.io.InvalidObjectException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 *
 * @version $Revision: 1.1 $ $Date: 2002/04/15 13:02:17 $
 */
public class Associativity implements Serializable
{

	/**
	 * The left type
	 */
	public static final int LEFT_TYPE = 0;

	/**
	 * The instance of the left type
	 */
	public static final Associativity LEFT = new Associativity(LEFT_TYPE, "left");

	/**
	 * The right type
	 */
	public static final int RIGHT_TYPE = 1;

	/**
	 * The instance of the right type
	 */
	public static final Associativity RIGHT = new Associativity(RIGHT_TYPE,
																						"right");

	/**
	 * The nonassoc type
	 */
	public static final int NONASSOC_TYPE = 2;

	/**
	 * The instance of the nonassoc type
	 */
	public static final Associativity NONASSOC = new Associativity(NONASSOC_TYPE,
																							 "nonassoc");

	private static Hashtable _memberTable = init();

	private int _type = -1;

	private String _stringValue = null;

	/**
	 *
	 *
	 * @param type       
	 * @param value      
	 */
	private Associativity(int type, String value)
	{
		super();
		_type = type;
		_stringValue = value;
	}

	/**
	 * Returns an enumeration of all possible instances of
	 * Associativity
	 */
	public static Enumeration enumerate()
	{
		return _memberTable.elements();
	}

	/**
	 * Returns the type of this Associativity
	 */
	public int getType()
	{
		return _type;
	}

	/**
	**/
	private static Hashtable init()
	{
		Hashtable members = new Hashtable();

		members.put("left", LEFT);
		members.put("right", RIGHT);
		members.put("nonassoc", NONASSOC);
		return members;
	}

	/**
	 * Returns the String representation of this Associativity
	 */
	public String toString()
	{
		return _stringValue;
	}

	/**
	 * Returns a new Associativity based on the given String value.
	 * @param string
	 */
	public static Associativity valueOf(String string)
	{
		Object obj = null;

		if (string != null)
			obj = _memberTable.get(string);
		if (obj == null)
		{
			String err = "'" + string + "' is not a valid Associativity";

			throw new IllegalArgumentException(err);
		}
		return (Associativity) obj;
	}

  private void writeObject(ObjectOutputStream out) throws IOException
  {
    out.defaultWriteObject();
    out.writeInt(_type);
    out.writeUTF(_stringValue);
  }

  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
  {
    in.defaultReadObject();
    _type = in.readInt();
    _stringValue = in.readUTF();
  }

  /** 
   * Deals with the issue of uniqueness when we are dealing with more
   * than one VM by adding the read symbol to the hash table, if it
   * isn't already there.
   */
  public Object readResolve() throws ObjectStreamException
  {
    if (!_memberTable.containsKey(_stringValue))
      throw new InvalidObjectException("'" + _stringValue + "' is not a valid Associativity");
    return (Associativity) (_memberTable.get(_stringValue));
  }
}
