/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.grammar.token.definition;

import java.io.Serializable;

import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.helpers.IntegerList;
import net.sourceforge.chaperon.parser.RegexAutomate;

/**
 * This class describes a regex alternation
 *
 * @author Stephan Michels
 * @version $Revision: 1.2 $ $Date: 2002/05/06 16:22:56 $
 */
public class Alternation extends DefinitionElementList implements Serializable, Cloneable
{

  public Alternation()
  {
  }

  /**
   * Convert the alternation to a string
   *
   * @return Content of the alternation
   */
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();

    for (int i = 0; i < getDefinitionElementCount(); i++)
    {
      if (i > 0)
        buffer.append("|");
      buffer.append(getDefinitionElement(i).toString());
    }
    return buffer.toString();
  }

  public Object clone() throws CloneNotSupportedException
  {
    Alternation clone = new Alternation();

    clone.setMinOccurs(getMinOccurs());
    clone.setMaxOccurs(getMaxOccurs());

    for (int i = 0; i < getDefinitionElementCount(); i++)
      clone.addDefinitionElement(getDefinitionElement(i));

    return clone;
  }
}
